﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    public class DrawningObject : IDrawningObject
    {
        private DrawningAirplane _airplane;

        public DrawningObject(DrawningAirplane airplane)
        {
            _airplane = airplane;
        }

        public float Step => _airplane?.Airplane?.Step ?? 0;

        public RectangleF GetCurrentPosition()
        {
            return _airplane.GetCurrentPosition();
        }

        public void MoveObject(Direction direction)
        {
            _airplane?.MoveTransport(direction);
        }

        public void SetObject(int x, int y, int width, int height)
        {
            _airplane?.SetPosition(x, y, width, height);
        }

        public void DrawObject(Graphics g)
        {
            _airplane.DrawTransport(g);
        }

        /// <summary>
        /// Получения самолета из объекта отрисовки
        /// </summary>
        /// <param name="drawningObject">The drawning object.</param>
        /// <returns></returns>
        public static explicit operator DrawningAirplane?(DrawningObject? drawningObject) => drawningObject?._airplane ?? null;

        public string GetInfo() => _airplane?.GetDataForSave() ?? string.Empty;

        public static IDrawningObject Create(string data) => new DrawningObject(data.CreateDrawningAirplane());

        public bool Equals(IDrawningObject? other)
        {
            if (other is not DrawningObject otherAirplane || 
                _airplane.DrawningEngines?.GetType()    != otherAirplane._airplane.DrawningEngines?.GetType() ||
                _airplane.DrawningEngines?.CountEngines != otherAirplane._airplane.DrawningEngines?.CountEngines)
            {
                return false;
            }
            var entity = _airplane.Airplane;
            var otherEntity = otherAirplane._airplane.Airplane;
            if (entity.GetType() != otherEntity.GetType() ||
                entity.Speed != otherEntity.Speed ||
                entity.Weight != otherEntity.Weight ||
                entity.BodyColor != otherEntity.BodyColor)
            {
                return false;
            }
            if (entity is EntityAirBomber entityAirBomber &&
                otherEntity is EntityAirBomber otherEntityAirBomber && (
                entityAirBomber.HasBombs != otherEntityAirBomber.HasBombs ||
                entityAirBomber.DopColor != otherEntityAirBomber.DopColor ||
                entityAirBomber.HasFuelTanks != otherEntityAirBomber.HasFuelTanks))
            {
                return false;
            }
            return true;
        }
    }
}
